#version 130 // GLSL verzia

uniform sampler3D base_tex_unit; // zakladna textura

uniform vec3 lightWoodColor; // svetla farba dreva
uniform vec3 darkWoodColor;  // tmava farba dreva

in vec4 pos; // poloha vrcholu v EYE space
in vec3 nor;
in vec3 dir_to_light,dir_to_eye;
in float dist;
in vec4 ambient,diffuse,specular;

in vec3 fragment_pos; // poloha fragmentu v OBJECT space

void main(void)
{
 vec4 amb=vec4(1.0),dif=vec4(0.0),spe=vec4(0.0);
 vec3 N,L,E,HV;
 float NdotL,att=0.0,spot;

 // pseudonahodne cislo z 3D perlin noise textury
 vec3 pnoise=vec3(texture3D(base_tex_unit,fragment_pos));
 // vrchol na pozicii fragment_pos + pnoise
 vec3 rnpos=fragment_pos+pnoise;
 // 2D vzdialenost vrcholu rnpos od 0,0,0 * pocet letokruhov
 float d=sqrt(rnpos.x*rnpos.x+rnpos.y*rnpos.y)*3.0;
 // fract(x) = x - floor(x), x je typu float => fract(x) = x - (int) x
 //    /  /  /
 //   /  /  /
 //  /  /  /
 // 0  1  2
 // namiesto 2.0 je pouzita hodnota 2.5 => ostrejsie prechody struktury dreva
 d=fract(d)*2.5; if (d>1.0) d=2.5-d;

 // farba pixelu
 // mix(x,y,a) = x * (1.0 - a) + y * a
 amb.rgb=mix(lightWoodColor,darkWoodColor,d);

 // sikme pruhy, 150.0 = frekvencia pruhov, 0.3 = intenzita pruhov
 d=fract((fragment_pos.x+fragment_pos.z)*150.0);
 if (d<0.5) amb.rgb+=lightWoodColor*d*0.3;

 // povodna farba z 3D perlin noise textury
 //amb.rgb=vec3(texture3D(base_tex_unit,fragment_pos));

 // ambient
 amb.rgb=clamp(amb.rgb,0.0,1.0);
 amb*=ambient;

 // normalizacia interpolovanych udajov
 N=normalize(nor);
 L=normalize(dir_to_light);

 NdotL=max(dot(N,L),0.0);
 if (NdotL>0.0) {
  spot=dot(gl_LightSource[0].spotDirection,-L);
  if (spot>gl_LightSource[0].spotCosCutoff) {
   // utlm
   att=pow(spot,gl_LightSource[0].spotExponent)/
       (gl_LightSource[0].constantAttenuation+
        gl_LightSource[0].linearAttenuation*dist+
        gl_LightSource[0].quadraticAttenuation*dist*dist);

   // diffuse
   dif=NdotL*diffuse;

   // specular
   E=normalize(dir_to_eye);
   HV=normalize(-reflect(L,N)); // half vector
   spe=pow(max(dot(E,HV),0.0),gl_FrontMaterial.shininess)*specular;
  }
 }

 // vysledna farba
 gl_FragColor=amb+att*(dif+spe);
}